'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var constants = require('../../common/constants.js');
var utils = require('./utils.js');

const toSocketIoMessagePayload = (id, data) => {
  return `${constants.constants.websocket.packetTypes.send}${id}${JSON.stringify(['socket.io-request', data])}`;
};
const isConnectionMessage = message => {
  return message[0] === constants.constants.websocket.packetTypes.initialize;
};
const isResponseMessage = message => {
  return message[0].length > 2 && message[0].startsWith(constants.constants.websocket.packetTypes.response);
};
const getRequestId = packetType => packetType.slice(2);
const isResponseMessageOf = sid => message => {
  return isResponseMessage(message) && sid === getRequestId(message[0]);
};
const isResponseOf = expected => {
  return ([sid]) => expected === sid;
}; // 430[{...}}] => 430, [{}]
// @see https://github.com/socketio/engine.io-protocol

const scrapboxDeserializer = event => {
  let packetType = '';
  let message = String(event.data);

  while (message.length) {
    const head = message[0];

    if (utils.isIntegerString(head)) {
      packetType += head;
      message = message.substr(1);
    } else {
      return [packetType, JSON.parse(message)];
    }
  }

  return [packetType, undefined];
};

exports.getRequestId = getRequestId;
exports.isConnectionMessage = isConnectionMessage;
exports.isResponseMessage = isResponseMessage;
exports.isResponseMessageOf = isResponseMessageOf;
exports.isResponseOf = isResponseOf;
exports.scrapboxDeserializer = scrapboxDeserializer;
exports.toSocketIoMessagePayload = toSocketIoMessagePayload;
//# sourceMappingURL=message.js.map
