'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var utils = require('../../common/utils.js');
var debugWebsocket = require('./debug-websocket.js');

const defaultDeserializer = message => message.data;

class RxWebSocket extends rxjs.Subject {
  #websocket;
  #open$;

  constructor(url, {
    protocols,
    clientOptions,
    deserializer = defaultDeserializer,
    debug = false
  }) {
    super();
    const websocket = utils.isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, clientOptions);

    if (debug) {
      debugWebsocket.debugWebsocket(websocket);
    }

    const open$ = rxjs.fromEvent(websocket, 'open').pipe(rxjs.take(1), rxjs.shareReplay());
    const error$ = rxjs.fromEvent(websocket, 'error').pipe(rxjs.take(1), rxjs.shareReplay());
    const close$ = rxjs.fromEvent(websocket, 'close').pipe(rxjs.take(1), operators.share());
    const message$ = rxjs.fromEvent(websocket, 'message').pipe(operators.takeUntil(close$), rxjs.shareReplay(1));
    rxjs.merge(open$.pipe(rxjs.ignoreElements()), message$.pipe(operators.map(deserializer)), error$.pipe(rxjs.mergeMap(rxjs.throwError)), close$.pipe(rxjs.ignoreElements())).subscribe(this);
    this.#open$ = open$;
    this.#websocket = websocket;
  }

  send(data) {
    this.#open$.subscribe(() => this.#websocket.send(data));
    return this;
  }

  close() {
    this.#websocket.close();
    this.unsubscribe();
  }

}

exports.RxWebSocket = RxWebSocket;
//# sourceMappingURL=websocket.js.map
