'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var constants = require('../common/constants.js');
var message = require('./internal/message.js');
var request = require('./internal/request.js');
var utils = require('./internal/utils.js');
var websocket = require('./internal/websocket.js');

class ScrapboxWebsocketHandler {
  #room = null;
  #sid = 0;
  #socket$;
  #close$ = new rxjs.Subject();

  constructor({
    token = '',
    debug
  }) {
    this.#socket$ = new rxjs.Observable(subscriber => {
      const socket = new websocket.RxWebSocket(constants.constants.websocket.endpoint, {
        clientOptions: {
          headers: {
            Origin: constants.constants.websocket.origin,
            Cookie: utils.getAuthCookieValue(token)
          }
        },
        deserializer: message.scrapboxDeserializer,
        debug
      });
      socket.pipe(operators.takeUntil(this.#close$), rxjs.filter(message.isConnectionMessage), // send ack
      rxjs.tap(() => {
        socket.send(constants.constants.websocket.packetTypes.connected);
      }), // set ping
      rxjs.tap(([, data]) => rxjs.interval(data.pingInterval).pipe(operators.takeUntil(socket)).subscribe(() => socket.send(constants.constants.websocket.packetTypes.ping))), rxjs.delayWhen(() => socket.pipe(operators.first(message.isResponseOf(constants.constants.websocket.packetTypes.connected)))), rxjs.mapTo(socket), operators.timeout(constants.constants.websocket.responseTimeout)).subscribe(subscriber);
      return () => socket.close();
    }).pipe(rxjs.shareReplay(1));
  }

  close() {
    this.#close$.next(undefined);
  }

  async commit({
    projectId,
    userId,
    pageId,
    parentId,
    changes
  }) {
    return rxjs.firstValueFrom(this.#join({
      projectId,
      pageId
    }).pipe(rxjs.concatMap(() => {
      return this.#send({
        method: 'commit',
        data: {
          kind: 'page',
          userId,
          projectId,
          pageId,
          parentId,
          changes: request.createChanges(changes, userId),
          cursor: null,
          freeze: true
        }
      });
    })));
  }

  #join({
    projectId,
    pageId
  }) {
    var _this$room, _this$room2;

    return ((_this$room = this.#room) === null || _this$room === void 0 ? void 0 : _this$room.projectId) === projectId && ((_this$room2 = this.#room) === null || _this$room2 === void 0 ? void 0 : _this$room2.pageId) === pageId ? rxjs.of({}) : this.#send({
      method: 'room:join',
      data: {
        pageId,
        projectId,
        projectUpdatesStream: false
      }
    }).pipe(rxjs.tap({
      complete: () => {
        this.#room = {
          projectId,
          pageId
        };
      }
    }));
  }

  #send(data) {
    const sid = String(this.#sid++);
    return this.#socket$.pipe(rxjs.take(1), rxjs.mergeMap(socket => socket.send(message.toSocketIoMessagePayload(sid, data)).pipe(operators.first(message.isResponseMessageOf(sid)), operators.timeout(constants.constants.websocket.responseTimeout))));
  }

}

exports.ScrapboxWebsocketHandler = ScrapboxWebsocketHandler;
//# sourceMappingURL=websocket-handler.js.map
