import { Subject, fromEvent, take, shareReplay, merge, ignoreElements, mergeMap, throwError } from 'rxjs';
import { share, takeUntil, map } from 'rxjs/operators';
import { isBrowser } from '../../common/utils.js';
import { debugWebsocket } from './debug-websocket.js';

const defaultDeserializer = (message) => message.data;
class RxWebSocket extends Subject {
    #websocket;
    #open$;
    constructor(url, { protocols, clientOptions, deserializer = defaultDeserializer, debug = false }) {
        super();
        const websocket = isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, clientOptions);
        if (debug) {
            debugWebsocket(websocket);
        }
        const open$ = fromEvent(websocket, 'open').pipe(take(1), shareReplay());
        const error$ = fromEvent(websocket, 'error').pipe(take(1), shareReplay());
        const close$ = fromEvent(websocket, 'close').pipe(take(1), share());
        const message$ = fromEvent(websocket, 'message').pipe(takeUntil(close$), shareReplay(1));
        merge(open$.pipe(ignoreElements()), message$.pipe(map(deserializer)), error$.pipe(mergeMap(throwError)), close$.pipe(ignoreElements())).subscribe(this);
        this.#open$ = open$;
        this.#websocket = websocket;
    }
    send(data) {
        this.#open$.subscribe(() => this.#websocket.send(data));
        return this;
    }
    close() {
        this.#websocket.close();
        this.unsubscribe();
    }
}

export { RxWebSocket };
//# sourceMappingURL=websocket.js.map
